{* PASOS VIDEO DRIVER
   DESIGNED FOR VGA: 640x480x16

   Author:       Rodichev Igor
   Optimized by: Voskov Alexey
*}

Unit Video;
Interface

uses dos;

procedure StartVGA;
procedure CloseVGA;

Procedure FontLoader(fontname: String);
function GetPixel(x,y:integer):byte;
procedure Bar2(x1,y1,x2,y2:integer;c:byte);
procedure Rectangle(x1,y1,x2,y2:integer;c:byte);
procedure Line(x1,y1,x2,y2:integer;c:byte);
procedure outtextxy(x,y:integer;s:string;c:byte);
procedure PutPix(x,y:word;color:byte);
function GetPix (x,y:word):byte;

procedure ShowMouse;
procedure HideMouse;
function MousePosition(var MouseX,MouseY:Word):Word;
procedure SetMousePosition(mousex, mousey:Word);

implementation

var RusFont: array[0..255, 0..16] of byte;

{ Set 640x480x16 video mode }
procedure StartVGA;
begin
  FontLoader('Vga866.fon');
  asm
  mov ah,00h
  mov al,12h
  int 10h
  end;
end;

{ Set text mode }
procedure CloseVGA;assembler;
asm
mov ah,00h
mov al,03h
int 10h
end;


{ Set pixel address - ax = x, bx = y }
procedure  VideoAddr; assembler;
asm
   cld
   { x *= 8 }
   mov     cl,3
   shr     ax,cl
   mov     di,ax {di - adr in video buf}
   { y *= 8 }
   mov     ax,80
   mul     bx
   add     di,ax {di - adr in video buf}
   { set ES to video buffer }
   mov     ax,0A000H
   mov     es,ax
end; {VideoAddr}

function  GetPix (x,y : word) : byte; assembler;
        asm
        mov     ax,x
        mov     bx,y
        call    VideoAddr
        mov     cx,x
        and     cx,7
        neg     cx
        add     cx,7            { CX - ??. ?no?uo o?ao }
        mov     dx,3CEH
        xor     bl,bl
        mov     ah,3
@1:     mov     al,4
        out     dx,al
        inc     dx
        mov     al,ah
        out     dx,al
        dec     dx
        mov     al,es:[di]
        shr     al,cl
        and     al,1
        xchg    cl,ah
        shl     al,cl
        xchg    cl,ah
        or      bl,al
        dec     ah
        jns     @1
        mov     al,bl
        end; {GetPix}

{***** DRAWS LONG HORIZONTAL LINE OR RECTANGLE *****}
{Input parameters:
 y: word - coordinate of first line
 x: word - coordinate of line beginning
 xend: word - coordinate of line end
 color: byte - color (0..15)
 rep: word - number of lines
}
procedure HorLine (y: word; x: word; xend: word; color: byte; rep: word);
var adrbegin, adrend: word;
var headmask, tailmask: byte;
var adrbase: word;
Begin
     headmask := 0;
     tailmask := 0;
     adrbase := y * 80;
        asm
        {***** Part 1. Initialization *****}
        { Set video seg }
        mov     ax,0A000H
        mov     es,ax
        { Generate color mask}
        mov     cx,255
        cmp     color,0
        jz      @2
        mov     cl,color        {CL - oN? n?? ?a???}
        dec     ch              {CH - na???N n?? ?a???}
@2:     { Set color mask }
        mov     ah,cl
        mov     dx,3C4H
        mov     al,2
        out     dx,ax

        { Calculate offset for beginning of cycle}
        mov     bx, x
        shr     bx, 3
        add     bx, adrbase
        mov     adrbegin, bx
        { Calculate offset for ending of cycle}
        mov     bx, xend
        shr     bx, 3
        add     bx, adrbase
        mov     adrend, bx

        {*********************************}
        {*** Special case - short line ***}
        mov     ax, adrbegin
        mov     bx, adrend
        cmp     ax, bx
        jne     @longline

        mov     di, adrbegin
        { Generate mask }
        mov     bx, x
        mov     dx, xend
        and     bx, 7
        and     dx, 7
        { Save CX and fill mask bits with '1'}
        push    cx
        mov     ax, $FF00
        {First address and left edge of mask }
        mov     cl, bl
        shl     ax, cl
        shr     ax, cl
        {Second address and right edge of mask }
        mov     cl, 7
        sub     cl, dl
        shr     ax, cl
        xor     al, al
        shl     ax, cl
        { And setup this mask }
        mov     dx,3CEH
        mov     al,8
        out     dx,ax
        pop     cx { Restore CX }
        { Write pixels }
        mov     si, rep
@sm_loop:
        cmp     si, 0
        je     @sml_end

        mov     al, ch
        xchg    es:[di],al

        add     di, 80
        dec     si
        jmp     @sm_loop
@sml_end:
       jmp     @nocycle
@longline:
        {***************************************}
        {***** Part 2. First byte - "head" *****}
        { Calculate mask for "head"}
        mov     bx, x
        and     bl, 7
        { Generate "head" mask }
        push    cx
        mov     ah, 255
        mov     cl, bl
        shr     ax, cl
        mov     headmask, ah {* SAVE HEAD MASK *}
        pop     cx
@noheadmask:

        {***** Part 3. Last byte - "tail" *****}
        { Calculate mask for "tail"}
        mov     bx, xend
        and     bl, 7
        { Generate "tail" mask }
        push    cx
        mov     ax, $FF00
        mov     cl, 7
        sub     cl, bl
        shl     ax, cl
        mov     tailmask, ah {* SAVE TAIL MASK *}
        pop     cx

        mov     ax, adrbegin
        mov     bx, adrend
        cmp     ax, bx
        ja     @nocycle
        {***************************************}

        {** BEGINNING OF BIG CYCLE (ONE ITERATION - ONE STRING) **}
        mov     si, rep {SI - counter of big cycle  }
@bigloop:
        cmp     si, 0      {Condition of exit}
        je      @nocycle

        {* Setup head mask *}
        mov     ah, headmask
        mov     dx,3CEH
        mov     al,8
        out     dx,ax
        { Address }
        mov     di, adrbegin
        { Write pixels }
        mov     al, ch
        xchg    es:[di],al

        {* Setup tail mask *}
        mov     ah, tailmask
        mov     dx,3CEH
        mov     al,8
        out     dx,ax
        { Address }
        mov     di, adrend
        { Write pixels }
        mov     al, ch
        xchg    es:[di],al

        {* Drawing of the central part consisting of 8-pixel blocks *}
        {BEGIN OF @llc2 CYCLE}
        {Initialization of counter - di}
        mov     di, adrbegin
        { Generate mask from x - 1 bit per pixel in layer }
        mov     ah, 255
        { And setup this mask }
        mov     dx,3CEH
        mov     al,8
        out     dx,ax
        {Begin of loop}
@llc2:  inc     di
        cmp     di,adrend
        je      @llc2_end
        { Write pixel}
        mov     al, ch
        xchg    es:[di],al

        jmp     @llc2
@llc2_end:
        {END OF @llc2 CYCLE}

        {* Go to the next string *}
        add     adrbegin, 80 {Beginning of the string - to the next line}
        add     adrend,   80 {Ending of the string    - to the next line}
        dec     si           {Counter of big cycle}
        jmp     @bigloop

        {** END OF BIG CYCLE (ONE ITERATION - ONE STRING) **}
@nocycle:
        {RESTORE OLD MODE}
        mov     ah,15
        mov     dx,3C4H
        mov     al,2
        out     dx,ax

        mov     ah,0FFH
        mov     dx,3CEH
        mov     al,8
        out     dx,ax

        end; {PutPix}
end;

procedure  PutPix (x,y : word; color : byte);
begin
        asm
        { Set video adr }
        mov     ax,x
        mov     bx,y
        call    VideoAddr
        { Generate mask from x - 1 bit per pixel in layer }
        mov     cl,byte ptr x {lower byte from x}
        and     cl,7          {leave lower half-byte}
        mov     ah,80H        {AH - initialize mask register}
        shr     ah,cl         {make mask}
        { Set up pixel mask - use only needed color layers }
        mov     dx,3CEH
        mov     al,8 {AL - video reg number, AH - layer mask }
        out     dx,ax

        mov     cx,15
        cmp     color,0
        jz      @1
        mov     cl,color        {CL - oN? n?? ?a???}
        dec     ch              {CH - na???N n?? ?a???}
@1:     {Clear byte - only into pixel mask}
        xor     al,al
        xchg    es:[di],al
        mov     ah,cl
        {Set up color mask
         AH - mask
         AL - number of register for mask}
        mov     dx,3C4H
        mov     al,2
        out     dx,ax

        {Draw pixel}
        mov     al,ch
        stosb

        mov     ah,15
        mov     dx,3C4H
        mov     al,2
        out     dx,ax

        mov     ah,0FFH
        mov     dx,3CEH
        mov     al,8
        out     dx,ax
        end; {PutPix}
end;

function GetPixel(x,y:integer):byte;
var c:byte;
begin
asm
mov ah,0dh
mov cx,x
mov dx,y
mov bh,00h
int 10h
mov c,al
end;
getpixel:=c;
end;


procedure Bar2(x1,y1,x2,y2:integer;c:byte);
var i,j,rep:integer;
var adrbase:integer;
begin
  rep := (y2-y1) + 1;
  HorLine(y1,x1,x2,0,rep);
  HorLine(y1,x1,x2,c,rep);
end;

procedure Line(x1,y1,x2,y2:integer;c:byte);
var i,x,y:integer;
    dx, dy, i1, i2, kx, ky: Integer;
    d: Integer; {Deviation }
    flag: Integer;
begin
     dy := y2-y1; dx := x2-x1;
     if dx = 0 then if dy = 0 then begin
        PutPix(x1,y1,c);
        Exit;
     end;
     kx := 1; {Step by x}
     ky := 1; {Step by y}

     {Vibor taktovoi osi}
     if dx < 0 then
     begin
          dx := -dx;
          kx := -1;
     end
     else if dx = 0 then kx := 0;

     if (dy < 0) then begin
        dy := -dy;
        ky := -1;
     end;

     if (dx < dy) then
     begin
          flag := 0;
          d := dx; dx := dy; dy := d;
     end
     else
         flag := 1;

     i1 := dy + dy; d := i1 - dx; i2 := d - dx;
     x := x1; y := y1;

     for i:=0 to (dx-1) do Begin
         PutPix(x, y, c);

         if flag = 1 then x := x + kx
         else             y := y + ky;

         if d < 0 then d := d + i1
         else
         Begin
              d := d + i2;
              if flag = 1 then y := y + ky
              else             x := x + kx;
         End;
         PutPix(x, y, c);

     End
end;


procedure Rectangle(x1,y1,x2,y2:integer;c:byte);
var i,i1:integer;
begin
for i:=x1 to x2 do begin
PutPix(i,y1,c);PutPix(i,y2,c);end;

for i1:=y1 to y2 do begin
PutPix(x1,i1,c);PutPix(x2,i1,c);end;
end;

Procedure FontLoader(fontname: String);
var f: Text; {Pointer to the file}
    i, n: Integer;
Begin
  assign(f, fontname);
  reset(f);

  for i := 0 to 255 do
  Begin
       readln(f, RusFont[i,0], RusFont[i,1], RusFont[i,2], RusFont[i,3],
                 RusFont[i,4], RusFont[i,5], RusFont[i,6], RusFont[i,7],
                 RusFont[i,8], RusFont[i,9], RusFont[i,10], RusFont[i,11],
                 RusFont[i,12], RusFont[i,13], RusFont[i,14], RusFont[i,15]);
  End;
  close(f);
End;


procedure outtextxy(x,y:integer;s:string;c:byte);
var i, j, k: integer;
var mask: byte;
begin
  for i:=1 to length(s) do {* Symbol by symbol *}
  begin
    {* Drawing of one symbol *}
    for j:=0 to 15 do {* Symb.string by Symb.string *}
    begin
      mask := 128;
      for k:=0 to 7 do {* Bit by bit *}
      begin
        if (mask AND RusFont[ord(s[i]),j] <> 0) then
          PutPix(i*9+k+x,j+y,c);
        mask := mask div 2;
      end;
    end;
  end;
end;

procedure ShowMouse;assembler;
asm
mov ax,1
int 33h
end;

procedure HideMouse;assembler;
asm
mov ax,2
int 33h
end;

function MousePosition(var MouseX,MouseY:Word):Word;
var x,y,s:word;
begin
asm
mov ax,3
int 33h
dec cx
dec dx
mov x,cx
mov y,dx
mov s,bx
end;
mousex:=x;mousey:=y;MousePosition:=s;
end;

procedure SetMousePosition(mousex,mousey:Word);assembler;
asm
mov ax,4
mov cx,mousex
mov dx,mousex
inc cx
inc dx
int 33h
end;

begin
end.